'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Input/output helpers.
 */
exports.default = {
  /**
   * Returns the contents of an entire file.
   * When no filename given, reads from STDIN.
   * @param  {String} filename
   * @return {String}
   */
  read: function read(filename) {
    if (filename) {
      return _fs2.default.readFileSync(filename).toString();
    } else {
      // NOTE: This does not work in Windows.
      // Haven't managed to figure out how to read STDIN
      // in proper cross-platform way.
      return _fs2.default.readFileSync('/dev/stdin').toString();
    }
  },


  /**
   * Writes the data to file.
   * When no filename given, writes to STDIN.
   * @param  {String} filename
   * @param  {String} data
   */
  write: function write(filename, data) {
    if (filename) {
      _fs2.default.writeFileSync(filename, data);
    } else {
      process.stdout.write(data);
    }
  }
};
module.exports = exports['default'];
