'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (ast, logger) {
  _traverser2.default.replace(ast, {
    enter: function enter(node, parent) {
      if (isFunctionConvertableToArrow(node, parent)) {
        if (hasArguments(node.body)) {
          logger.warn(node, 'Can not use arguments in arrow function', 'arrow');
          return;
        }
        return functionToArrow(node);
      }

      var _matchBoundFunction = matchBoundFunction(node);

      var func = _matchBoundFunction.func;

      if (func) {
        return functionToArrow(func);
      }
    }
  });
};

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _traverser = require('../traverser');

var _traverser2 = _interopRequireDefault(_traverser);

var _arrowFunctionExpression = require('../syntax/arrow-function-expression');

var _arrowFunctionExpression2 = _interopRequireDefault(_arrowFunctionExpression);

var _matchesAst = require('../utils/matches-ast');

var _copyComments = require('../utils/copy-comments');

var _copyComments2 = _interopRequireDefault(_copyComments);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isFunctionConvertableToArrow(node, parent) {
  return node.type === 'FunctionExpression' && parent.type !== 'Property' && parent.type !== 'MethodDefinition' && !node.id && !node.generator && !hasThis(node.body);
}

// Matches: function(){}.bind(this)
var matchBoundFunction = (0, _matchesAst.matchesAst)({
  type: 'CallExpression',
  callee: {
    type: 'MemberExpression',
    computed: false,
    object: (0, _matchesAst.extract)('func', {
      type: 'FunctionExpression',
      id: null,
      body: function body(_body) {
        return !hasArguments(_body);
      },
      generator: false
    }),
    property: {
      type: 'Identifier',
      name: 'bind'
    }
  },
  arguments: [{
    type: 'ThisExpression'
  }]
});

function hasThis(ast) {
  return hasInFunctionBody(ast, { type: 'ThisExpression' });
}

function hasArguments(ast) {
  return hasInFunctionBody(ast, { type: 'Identifier', name: 'arguments' });
}

// Returns true when pattern matches any node in given function body,
// excluding any nested functions
function hasInFunctionBody(ast, pattern) {
  var predicate = _lodash2.default.matches(pattern);
  var found = false;

  _traverser2.default.traverse(ast, {
    enter: function enter(node) {
      if (predicate(node)) {
        found = true;
        this.break();
      }
      if (node.type === 'FunctionExpression' || node.type === 'FunctionDeclaration') {
        this.skip();
      }
    }
  });

  return found;
}

function functionToArrow(func) {
  return new _arrowFunctionExpression2.default({
    body: extractArrowBody(func.body),
    params: func.params,
    defaults: func.defaults,
    rest: func.rest
  });
}

function extractArrowBody(block) {
  var returnStat = block.body[0];
  if (returnStat && returnStat.type === 'ReturnStatement') {
    var returnVal = returnStat.argument;
    // preserve return statement comments
    (0, _copyComments2.default)({ from: returnStat, to: returnVal });
    return returnVal;
  } else {
    return block;
  }
}
module.exports = exports['default'];
