'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (ast, logger) {
  _traverser2.default.replace(ast, {
    enter: function enter(node, parent) {
      if (isVarWithRequireCalls(node)) {
        if (parent.type !== 'Program') {
          logger.warn(node, 'import can only be at root level', 'commonjs');
          return;
        }

        (0, _multiReplaceStatement2.default)({
          parent: parent,
          node: node,
          replacements: node.declarations.map(function (dec) {
            return varToImport(dec, node.kind);
          }),
          preserveComments: true
        });
      }
    }
  });
};

var _traverser = require('../../traverser');

var _traverser2 = _interopRequireDefault(_traverser);

var _isString = require('../../utils/is-string');

var _isString2 = _interopRequireDefault(_isString);

var _matchesAst = require('../../utils/matches-ast');

var _multiReplaceStatement = require('../../utils/multi-replace-statement');

var _multiReplaceStatement2 = _interopRequireDefault(_multiReplaceStatement);

var _importDeclaration = require('../../syntax/import-declaration');

var _importDeclaration2 = _interopRequireDefault(_importDeclaration);

var _importSpecifier = require('../../syntax/import-specifier');

var _importSpecifier2 = _interopRequireDefault(_importSpecifier);

var _importDefaultSpecifier = require('../../syntax/import-default-specifier');

var _importDefaultSpecifier2 = _interopRequireDefault(_importDefaultSpecifier);

var _variableDeclaration = require('../../syntax/variable-declaration');

var _variableDeclaration2 = _interopRequireDefault(_variableDeclaration);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Converts VariableDeclarator to ImportDeclaration when we recognize it
// as such, otherwise converts it to full VariableDeclaration (of original kind).
function varToImport(dec, kind) {
  var m = void 0;
  if (m = matchRequire(dec)) {
    if (m.id.type === 'ObjectPattern') {
      return patternToNamedImport(m);
    } else if (m.id.type === 'Identifier') {
      return identifierToDefaultImport(m);
    }
  } else if (m = matchRequireWithProperty(dec)) {
    return propertyToNamedImport(m);
  } else {
    return new _variableDeclaration2.default(kind, [dec]);
  }
}

function patternToNamedImport(_ref) {
  var id = _ref.id;
  var sources = _ref.sources;

  return new _importDeclaration2.default({
    specifiers: id.properties.map(function (_ref2) {
      var key = _ref2.key;
      var value = _ref2.value;

      return new _importSpecifier2.default({
        local: value,
        imported: key
      });
    }),
    source: sources[0]
  });
}

function identifierToDefaultImport(_ref3) {
  var id = _ref3.id;
  var sources = _ref3.sources;

  return new _importDeclaration2.default({
    specifiers: [new _importDefaultSpecifier2.default(id)],
    source: sources[0]
  });
}

function propertyToNamedImport(_ref4) {
  var id = _ref4.id;
  var property = _ref4.property;
  var sources = _ref4.sources;

  return new _importDeclaration2.default({
    specifiers: [new _importSpecifier2.default({ local: id, imported: property })],
    source: sources[0]
  });
}

function isVarWithRequireCalls(node) {
  return node.type === 'VariableDeclaration' && node.declarations.some(function (dec) {
    return matchRequire(dec) || matchRequireWithProperty(dec);
  });
}

var isIdentifier = (0, _matchesAst.matchesAst)({
  type: 'Identifier'
});

// matches Property with Identifier key and value (possibly shorthand)
var isSimpleProperty = (0, _matchesAst.matchesAst)({
  type: 'Property',
  key: isIdentifier,
  computed: false,
  value: isIdentifier
});

// matches: {a, b: myB, c, ...}
var isObjectPattern = (0, _matchesAst.matchesAst)({
  type: 'ObjectPattern',
  properties: function properties(props) {
    return props.every(isSimpleProperty);
  }
});

// matches: require(<source>)
var matchRequireCall = (0, _matchesAst.matchesAst)({
  type: 'CallExpression',
  callee: {
    type: 'Identifier',
    name: 'require'
  },
  arguments: (0, _matchesAst.extract)('sources', function (args) {
    return args.length === 1 && (0, _isString2.default)(args[0]);
  })
});

// Matches: <id> = require(<source>);
var matchRequire = (0, _matchesAst.matchesAst)({
  type: 'VariableDeclarator',
  id: (0, _matchesAst.extract)('id', function (id) {
    return isIdentifier(id) || isObjectPattern(id);
  }),
  init: matchRequireCall
});

// Matches: <id> = require(<source>).<property>;
var matchRequireWithProperty = (0, _matchesAst.matchesAst)({
  type: 'VariableDeclarator',
  id: (0, _matchesAst.extract)('id', isIdentifier),
  init: {
    type: 'MemberExpression',
    computed: false,
    object: matchRequireCall,
    property: (0, _matchesAst.extract)('property', {
      type: 'Identifier'
    })
  }
});
module.exports = exports['default'];
