'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

// Transforms


require('babel-polyfill');

var _recast = require('recast');

var _recast2 = _interopRequireDefault(_recast);

var _parser = require('./parser');

var _parser2 = _interopRequireDefault(_parser);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _class = require('./transform/class');

var _class2 = _interopRequireDefault(_class);

var _template = require('./transform/template');

var _template2 = _interopRequireDefault(_template);

var _arrow = require('./transform/arrow');

var _arrow2 = _interopRequireDefault(_arrow);

var _let = require('./transform/let');

var _let2 = _interopRequireDefault(_let);

var _defaultParam = require('./transform/default-param');

var _defaultParam2 = _interopRequireDefault(_defaultParam);

var _argSpread = require('./transform/arg-spread');

var _argSpread2 = _interopRequireDefault(_argSpread);

var _objMethod = require('./transform/obj-method');

var _objMethod2 = _interopRequireDefault(_objMethod);

var _objShorthand = require('./transform/obj-shorthand');

var _objShorthand2 = _interopRequireDefault(_objShorthand);

var _noStrict = require('./transform/no-strict');

var _noStrict2 = _interopRequireDefault(_noStrict);

var _commonjs = require('./transform/commonjs');

var _commonjs2 = _interopRequireDefault(_commonjs);

var _exponent = require('./transform/exponent');

var _exponent2 = _interopRequireDefault(_exponent);

var _multiVar = require('./transform/multi-var');

var _multiVar2 = _interopRequireDefault(_multiVar);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var transformsMap = {
  'class': _class2.default,
  'template': _template2.default,
  'arrow': _arrow2.default,
  'let': _let2.default,
  'default-param': _defaultParam2.default,
  'arg-spread': _argSpread2.default,
  'obj-method': _objMethod2.default,
  'obj-shorthand': _objShorthand2.default,
  'no-strict': _noStrict2.default,
  'commonjs': _commonjs2.default,
  'exponent': _exponent2.default,
  'multi-var': _multiVar2.default
};

/**
 * Runs transforms on code.
 */

var Transformer = function () {
  /**
   * @param {String[]} transformNames List of transforms to enable
   */
  function Transformer() {
    var transformNames = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

    _classCallCheck(this, Transformer);

    this.transforms = transformNames.map(function (name) {
      return transformsMap[name];
    });
  }

  /**
   * Tranforms code using all configured transforms.
   *
   * @param {String} code Input ES5 code
   * @return {Object} Output ES6 code
   */


  _createClass(Transformer, [{
    key: 'run',
    value: function run(code) {
      var logger = new _logger2.default();

      return {
        code: this.applyAllTransforms(code, logger),
        warnings: logger.getWarnings()
      };
    }
  }, {
    key: 'applyAllTransforms',
    value: function applyAllTransforms(code, logger) {
      var _this = this;

      return this.ignoringHashBangComment(code, function (js) {
        var ast = _recast2.default.parse(js, { parser: _parser2.default });

        _this.transforms.forEach(function (transformer) {
          transformer(ast.program, logger);
        });

        return _recast2.default.print(ast).code;
      });
    }

    // strips hashBang comment,
    // invokes callback with normal js,
    // then re-adds the hashBang comment back

  }, {
    key: 'ignoringHashBangComment',
    value: function ignoringHashBangComment(code, callback) {
      var _code$match = code.match(/^(\s*#!.*?\r?\n|)([\s\S]*)$/);

      var _code$match2 = _slicedToArray(_code$match, 3);

      var /* all */hashBang = _code$match2[1];
      var js = _code$match2[2];

      return hashBang + callback(js);
    }
  }]);

  return Transformer;
}();

exports.default = Transformer;
module.exports = exports['default'];
