'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _estraverse = require('estraverse');

var _estraverse2 = _interopRequireDefault(_estraverse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// JSX AST types, as documented in:
// https://github.com/facebook/jsx/blob/master/AST.md
var jsxExtensions = {
  keys: {
    JSXIdentifier: [],
    JSXMemberExpression: ['object', 'property'],
    JSXNamespacedName: ['namespace', 'name'],
    JSXEmptyExpression: [],
    JSXExpressionContainer: ['expression'],
    JSXOpeningElement: ['name', 'attributes'],
    JSXClosingElement: ['name'],
    JSXAttribute: ['name', 'value'],
    JSXSpreadAttribute: ['argument'],
    JSXElement: ['openingElement', 'closingElement', 'children']
  }
};

/**
 * Proxy for ESTraverse.
 * Providing a single place to easily extend its functionality.
 *
 * Exposes the exact same API as ESTraverse.
 */
exports.default = {
  traverse: function traverse(tree, cfg) {
    return _estraverse2.default.traverse(tree, _lodash2.default.assign(cfg, jsxExtensions));
  },
  replace: function replace(tree, cfg) {
    return _estraverse2.default.replace(tree, _lodash2.default.assign(cfg, jsxExtensions));
  },

  VisitorOption: _estraverse2.default.VisitorOption
};
module.exports = exports['default'];
